@@ -1,4 +1,4 @@
-#' Apply the Exploratory Graph Analysis technique
+#' Applies the Exploratory Graph Analysis technique
 #'
 #' Estimates the number of dimensions of a given dataset/instrument
 #' using graphical lasso (\code{\link{EBICglasso.qgraph}}) or the
@@ -6,6 +6,8 @@
 #' method and the walktrap community detection algorithm (\code{\link[igraph]{cluster_walktrap}}).
 #' The glasso regularization parameter is set via EBIC.
 #'
+#' This algorithm includes checking for whether the data is unidimensional.
+#'
 #' @param data A dataframe with the variables to be used in the analysis or a correlation matrix.
 #' If the data used is a correlation matrix, the argument \code{n} will need to be specified.
 #'
@@ -31,6 +33,19 @@
 #'
 #' }
 #'
+#' @param algorithm A string indicating the algorithm to use.
+#' Current options are:
+#'
+#' \itemize{
+#'
+#' \item{\strong{\code{walktrap}}}
+#' {Computes the Walktrap algorithm using \code{\link[igraph]{cluster_walktrap}}}
+#'
+#' \item{\strong{\code{louvain}}}
+#' {Computes the Walktrap algorithm using \code{\link[igraph]{cluster_louvain}}}
+#'
+#' }
+#'
 #' @param steps Number of steps to be used in \code{\link[igraph]{cluster_walktrap}} algorithm.
 #' Defaults to 4.
 #'
@@ -45,6 +60,19 @@
 #'
 #' @author Hudson F. Golino <hfg9s at virginia.edu>, Alexander P. Christensen <alexpaulchristensen at gmail.com>, Maria Dolores Nieto <acinodam at gmail.com> and Luis E. Garrido <garrido.luiseduardo at gmail.com>
 #'
+#' @return Returns a list containing:
+#'
+#' \item{network}{A symmetric network estimated using either the
+#' \code{\link{EBICglasso.qgraph}} or \code{\link[NetworkToolbox]{TMFG}}}
+#'
+#' \item{wc}{A vector representing the community (dimension) membership
+#' of each node in the network. \code{NA} values mean that the node
+#' was disconnected from the network}
+#'
+#' \item{n.dim}{A scalar of how many total dimensions were identified in the network}
+#'
+#' \item{cor.data}{The zero-order correlation matrix}
+#'
 #' @examples
 #'
 #' \donttest{
@@ -70,6 +98,7 @@
 #' #plot
 #' plot(ega.intel)
 #' }
+#'
 #' @seealso \code{\link{bootEGA}} to investigate the stability of EGA's estimation via bootstrap
 #' and \code{\link{CFA}} to verify the fit of the structure suggested by EGA using confirmatory factor analysis.
 #'
@@ -84,188 +113,154 @@
 #' \emph{Intelligence}, \emph{62}, 54-70.
 #' doi: \href{https://doi.org/10.1016/j.intell.2017.02.007}{j.intell.2017.02.007}
 #'
-#' Golino, H., Shi, D., Garrido, L. E., Christensen, A. P., Nieto, M. D., Sadana, R., & Thiyagarajan, J. A. (2018).
+#' Golino, H., Shi, D., Christensen, A. P., Garrido, L. E., Nieto, M. D., Sadana, R., & Thiyagarajan, J. A. (in press).
 #' Investigating the performance of Exploratory Graph Analysis and traditional techniques to identify the number of latent factors: A simulation and tutorial.
-#' \emph{PsyArXiv}.
+#' \emph{Psychological Methods}.
 #' doi: \href{https://psyarxiv.com/gzcre/}{10.31234/osf.io/gzcre}
 #'
-#' @importFrom iterators iter nextElem
 #' @importFrom stats cor rnorm runif na.omit
 #'
 #' @export
 #'
-# EGA default function - 05/30/2019
+# Updated 04.02.2020
 ## EGA Function to detect unidimensionality:
-
-EGA <- function (data, model = c("glasso", "TMFG"), plot.EGA = TRUE, n = NULL,
+EGA <- function (data, model = c("glasso", "TMFG"),
+                 algorithm = c("walktrap", "louvain"),
+                 plot.EGA = TRUE, n = NULL,
                  steps = 4, nvar = 4, nfact = 1, load = 0.70, ...) {
 
-  #-------------------------------------------------------------------------
-  ## CHECKING DATA STRUCTURE
-  #-------------------------------------------------------------------------
-  if(nrow(data)!=ncol(data)){
-    data <- as.data.frame(data)
+  ##################################
+  #### DATA SIMULATION FUNCTION ####
+  ##################################
+
+  sim.func <- function(data, nvar, nfact, load)
+  {
+    # Check for unidimensional structure
+    ## Set up data simulation
     n <- nrow(data)
-  }
-
-  if (nrow(data) == ncol(data)){
-    cor.data <- data
-    if (missing(model)) {
-      model = "glasso"
-    }
-    else {
-      model = match.arg(model)
-    }
-    if (model == "glasso") {
-      gamma.values <- c(0.50, 0.25, 0)
-      gvals <- iterators::iter(gamma.values)
-      repeat {
-        estimated.network <- EBICglasso.qgraph(data = cor.data,
-                                               n = n,
-                                               lambda.min.ratio = 0.1,
-                                               returnAllResults = FALSE,
-                                               gamma = iterators::nextElem(gvals), ...)
-        if (any(NetworkToolbox::strength(estimated.network)>0)) {
-          break
-        }
-      }
-    }
-    else if (model == "TMFG") {
-      estimated.network <- NetworkToolbox::TMFG(cor.data, ...)$A
-    }
-    graph <- NetworkToolbox::convert2igraph(abs(estimated.network))
-    wc <- igraph::walktrap.community(graph, steps = steps)
-    names(wc$membership) <- colnames(data)
-    n.dim <- max(wc$membership)
-    wc <- wc$membership
-  } else{
-
-
-    #-------------------------------------------------------------------------
-    ## DATA GENERATION
-    #-------------------------------------------------------------------------
-
-    nvar <- nvar
-    nfac <- nfact
-    n <- nrow(data)
-    load <- load
     corf <- 0
-    J       = nvar*nfac
+    J <- nvar*nfact
     sdcross = 0
 
     ## GENERATE SAMPLE DATA MATRIX
     check.eig <- TRUE
     check.com <- TRUE
 
-    while(check.eig == TRUE|check.com == TRUE){
+    while(check.eig == TRUE|check.com == TRUE)
+    {
+      SATF = matrix(0, J, nfact)
 
-      SATF = matrix(0, J, nfac)
-
-      for(j in 1:nfac){
+      for(j in 1:nfact)
+      {
         SATF[(j*nvar-nvar+1):(j*nvar),j]<-runif(nvar, load-.10, load+.10)
-        if(nfac>1){
-          CROSS.L = apply(as.matrix(SATF[(j*nvar-nvar+1+2):(j*nvar),-c(j)]), ## Generate cross-loadings
-                          2,
-                          function(x) rnorm((nvar-2), 0, sdcross))
-          SATF[(j*nvar-nvar+1+2):(j*nvar),-c(j)] = CROSS.L
+
+        if(nfact>1)
+        {
+          CROSS.L <- apply(as.matrix(SATF[(j*nvar-nvar+1+2):(j*nvar),-c(j)]), 2, function(x) rnorm((nvar-2), 0, sdcross))
+
+          SATF[(j*nvar-nvar+1+2):(j*nvar),-c(j)] <- CROSS.L
         }
       }
-      SATF # Population factor loading matrix with cross-loadings and marker items
 
-      FCOR      = matrix(corf, nfac, nfac); diag(FCOR)<-1 ## Factor correlation matrix
+      #SATF # Population factor loading matrix with cross-loadings and marker items
+
+      FCOR      = matrix(corf, nfact, nfact); diag(FCOR)<-1 ## Factor correlation matrix
       R         = SATF%*%FCOR%*%t(SATF)                          ## Rr
       check.com = any(diag(R) > .90)                                  ## Check communalities values
       diag(R)   = 1                                                                    ## Insert ones in the diagonal of Rr
-      R                                                                                       ## Rp
+      #R                                                                                       ## Rp
       check.eig = any(eigen(R)$values <= 0)                      ## Check eigenvalues
     }
 
     U = chol(R)                                                                       ## Cholesky decomposition of Rp
     Z = mvtnorm::rmvnorm(n, sigma = diag(J))                                  ## Obtain sample matrix of continuous variables
     X = Z%*%U
-    X <-as.data.frame(X)
-    colnames(X) <- paste0("V", 1:ncol(X))
+    colnames(X) <- paste0("X", 1:ncol(X))
 
-    data.sim <- data.frame(X, data)
+    data.sim <- cbind(X, data)
+
+    return(data.sim)
+  }
+
+  ##################################
+  #### DATA SIMULATION FUNCTION ####
+  ##################################
+
+  #### MISSING ARGUMENTS HANDLING ####
+
+  if(missing(model))
+  {model <- "glasso"
+  }else{model <- match.arg(model)}
+
+  if(missing(algorithm))
+  {algorithm <- "walktrap"
+  }else{algorithm <- match.arg(algorithm)}
+
+  #### MISSING ARGUMENTS HANDLING ####
+
+  # Check for data or correlation matrix
+  if(nrow(data) == ncol(data))
+  {
+    # Multidimensional correlation result
+    multi.cor.res <- EGA.estimate(data = data, model = model, algorithm = algorithm, steps = steps, n = n, ...)
+
+    # Unidimensional correlation result
+    uni.data <- MASS::mvrnorm(n = n, mu = rep(0, ncol(data)), Sigma = multi.cor.res$cor.data)
+    sim.data <- sim.func(data = uni.data, nvar = nvar, nfact = nfact, load = load)
+    uni.cor.res <- suppressMessages(EGA.estimate(data = sim.data, model = model, algorithm = algorithm, steps = steps, n = n, ...))
+
+    # Set up results
+    if(uni.cor.res$n.dim <= nfact + 1)
+    {
+      n.dim <- uni.cor.res$n.dim
+      cor.data <- uni.cor.res$cor.data[-c(1:(nvar*nfact)),-c(1:(nvar*nfact))]
+      estimated.network <- uni.cor.res$network[-c(1:(nvar*nfact)),-c(1:(nvar*nfact))]
+      wc <- uni.cor.res$wc[-c(1:(nvar*nfact))]
+    }else{
+      n.dim <- multi.cor.res$n.dim
+      cor.data <- multi.cor.res$cor.data
+      estimated.network <- multi.cor.res$network
+      wc <- multi.cor.res$wc
+    }
+
+  }else{
+
+    # Convert to data frame
+    data <- as.data.frame(data)
 
     #-------------------------------------------------------------------------
-    ## EGA WITH SIMULATED DATA + ORIGINAL DATA
+    ## EGA WITH SIMULATED DATA + ORIGINAL DATA (UNIDIMENSIONALITY CHECK)
     #-------------------------------------------------------------------------
 
-    if (missing(model)) {
-      model = "glasso"
-    }
-    else {
-      model = match.arg(model)
-    }
+    n <- nrow(data)
 
-    cor.data.sim <- qgraph::cor_auto(data.sim)
+    data.sim <- sim.func(data = data, nvar = nvar, nfact = nfact, load = load)
 
-    if (model == "glasso") {
-      gamma.values <- c(0.50, 0.25, 0)
-      gvals <- iterators::iter(gamma.values)
-      repeat {
-        estimated.network.sim <- EBICglasso.qgraph(data = cor.data.sim,
-                                                   n = n,
-                                                   lambda.min.ratio = 0.1,
-                                                   returnAllResults = FALSE,
-                                                   gamma = iterators::nextElem(gvals), ...);
-        if (all(NetworkToolbox::strength(estimated.network.sim)>0)) {
-          break
-        }
-      }
-    }
-    else if (model == "TMFG") {
-      estimated.network.sim <- NetworkToolbox::TMFG(cor.data.sim, ...)$A
-    }
-    graph.sim <- NetworkToolbox::convert2igraph(abs(estimated.network.sim))
-    wc.sim <- igraph::walktrap.community(graph.sim, steps = steps)
-    names(wc.sim$membership) <- colnames(data.sim)
-    n.dim.sim <- max(wc.sim$membership)
-    if(n.dim.sim <= (nfact+1)){
-      n.dim <- n.dim.sim
-      cor.data <- cor.data.sim[-c(1:(nvar*nfact)),-c(1:(nvar*nfact))]
-      estimated.network <- estimated.network.sim[-c(1:(nvar*nfact)),-c(1:(nvar*nfact))]
-      wc <- wc.sim$membership[-c(1:(nvar*nfact))]
-    }
-    #-------------------------------------------------------------------------
-    ## TRADITIONAL EGA (IF NUMBER OF FACTORS > 2)
-    #-------------------------------------------------------------------------
+    uni.res <- EGA.estimate(data.sim, model = model, algorithm = algorithm, steps = steps, n = n, ...)
 
-    else{
+    if(uni.res$n.dim <= nfact + 1)
+    {
+      n.dim <- uni.res$n.dim
+      cor.data <- uni.res$cor.data[-c(1:(nvar*nfact)),-c(1:(nvar*nfact))]
+      estimated.network <- uni.res$network[-c(1:(nvar*nfact)),-c(1:(nvar*nfact))]
+      wc <- uni.res$wc[-c(1:(nvar*nfact))]
+    }else{
 
-      cor.data <- cor.data.sim[-c(1:nvar),-c(1:nvar)]
+      #-------------------------------------------------------------------------
+      ## TRADITIONAL EGA (IF NUMBER OF FACTORS > 2)
+      #-------------------------------------------------------------------------
 
-      if (missing(model)) {
-        model = "glasso"
-      }
-      else {
-        model = match.arg(model)
-      }
-      if (model == "glasso") {
-        gamma.values <- c(0.50, 0.25, 0)
-        gvals <- iterators::iter(gamma.values)
-        repeat {
-          estimated.network <- EBICglasso.qgraph(data = cor.data,
-                                                 n = n,
-                                                 lambda.min.ratio = 0.1,
-                                                 returnAllResults = FALSE,
-                                                 gamma = iterators::nextElem(gvals), ...)
-          if (all(NetworkToolbox::strength(estimated.network)>0)) {
-            break
-          }
-        }
-      }
-      else if (model == "TMFG") {
-        estimated.network <- NetworkToolbox::TMFG(cor.data, ...)$A
-      }
-      graph <- NetworkToolbox::convert2igraph(abs(estimated.network))
-      wc <- igraph::walktrap.community(graph, steps = steps)
-      names(wc$membership) <- colnames(data)
-      n.dim <- max(wc$membership)
-      wc <- wc$membership
+      cor.data <- uni.res$cor.data[-c(1:(nvar*nfact)),-c(1:(nvar*nfact))]
+
+      multi.res <- suppressMessages(EGA.estimate(cor.data, model = model, algorithm = algorithm, steps = steps, n = n, ...))
+
+      n.dim <- multi.res$n.dim
+      cor.data <- multi.res$cor.data
+      estimated.network <- multi.res$network
+      wc <- multi.res$wc
     }
   }
+
   a <- list()
   # Returning only communities that have at least two items:
   if(length(unique(wc))>1){
@@ -276,6 +271,7 @@ EGA <- function (data, model = c("glasso", "TMFG"), plot.EGA = TRUE, n = NULL,
   }else{
     a$n.dim <- length(unique(wc))
   }
+
   a$correlation <- cor.data
   a$network <- estimated.network
   a$wc <- wc
@@ -290,7 +286,9 @@ EGA <- function (data, model = c("glasso", "TMFG"), plot.EGA = TRUE, n = NULL,
       plot.ega <- qgraph::qgraph(a$network, layout = "spring",
                                  vsize = 6, groups = as.factor(a$wc), label.prop = 1, legend = TRUE)
     }
-  }
+  }else{plot.ega <- qgraph::qgraph(a$network, DoNotPlot = TRUE)}
+
+  row.names(a$dim.variables) <- plot.ega$graphAttributes$Nodes$labels[match(row.names(a$dim.variables), names(plot.ega$graphAttributes$Nodes$labels))]
 
   a$EGA.type <- ifelse(a$n.dim <= 2, "Unidimensional EGA", "Traditional EGA")
   class(a) <- "EGA"
